require "import"
import "android.widget.*"
import "android.content.Context"
import "android.media.AudioManager"
import "android.content.Intent"
import "android.net.Uri"
import "android.os.Vibrator"
import "cjson"
import "com.androlua.Http"

-- App Information
appName = "Advanced Volume Controller"
developerName = "Sandeep Ghimire"
apiUrl = "https://darkness.ashlynn.workers.dev/chat/?model=gpt-4o-mini&prompt="

-- Global Variables
local tickSoundEnabled = true
local vibrationEnabled = true
local vibrator = this.getSystemService(Context.VIBRATOR_SERVICE)

-- Play tick sound on open
if tickSoundEnabled then
    service.playSoundTick()
end

-- Function to encode URL
function urlEncode(s)
    return (s:gsub("[^%w_%-.~]", function(c)
        return string.format("%%%02X", string.byte(c))
    end))
end

-- Function to control audio streams
function createVolumeSlider(audioStream, label)
    local audioManager = this.getSystemService(Context.AUDIO_SERVICE)
    local maxVolume = audioManager.getStreamMaxVolume(audioStream)
    local currentVolume = audioManager.getStreamVolume(audioStream)

    local layout = {
        LinearLayout,
        orientation = "horizontal",
        layout_width = "fill",
        layout_height = "wrap",
        padding = "8dp",
        {
            TextView,
            text = label,
            layout_width = "0dp",
            layout_weight = 1,
            layout_height = "wrap",
        },
        {
            SeekBar,
            id = label .. "Slider",
            layout_width = "0dp",
            layout_weight = 2,
            layout_height = "wrap",
            max = maxVolume,
            progress = currentVolume,
            onProgressChanged = function(seekBar, progress, fromUser)
                if fromUser then
                    audioManager.setStreamVolume(audioStream, progress, 0)
                end
            end,
        },
    }
    return layout
end

-- Main Layout
mainLayout = {
    ScrollView,
    layout_width = "fill",
    layout_height = "fill",
    {
        LinearLayout,
        orientation = "vertical",
        layout_width = "fill",
        layout_height = "fill",
        padding = "8dp",
        {
            TextView,
            text = "Welcome to the Advanced Volume Controller!",
            layout_width = "fill",
            layout_height = "wrap",
            gravity = "center",
            textSize = 20,
        },
        createVolumeSlider(AudioManager.STREAM_MUSIC, "Music"),
        createVolumeSlider(AudioManager.STREAM_RING, "Ringtone"),
        createVolumeSlider(AudioManager.STREAM_ALARM, "Alarm"),
        createVolumeSlider(AudioManager.STREAM_NOTIFICATION, "Notification"),
        createVolumeSlider(AudioManager.STREAM_VOICE_CALL, "Call"),
        {
            Button,
            text = "More Options",
            layout_width = "fill",
            layout_height = "wrap",
            onClick = function()
                showMoreOptionsDialog()
            end,
        },
    },
}

-- Display Main Layout
dlg = LuaDialog(this)
dlg.View = loadlayout(mainLayout)
dlg.setTitle(appName)
dlg.setButton("Close", nil)
dlg.show()

-- Function to show More Options dialog
function showMoreOptionsDialog()
    local moreOptionsLayout = {
        LinearLayout,
        orientation = "vertical",
        layout_width = "fill",
        layout_height = "fill",
        padding = "8dp",
        {
            Button,
            text = "Chat with AI",
            layout_width = "fill",
            layout_height = "wrap",
            onClick = function()
                openChatDialog()
            end,
        },
        {
            Button,
            text = "About",
            layout_width = "fill",
            layout_height = "wrap",
            onClick = function()
                showAboutDialog()
            end,
        },
        {
            Button,
            text = "Settings",
            layout_width = "fill",
            layout_height = "wrap",
            onClick = function()
                showSettingsDialog()
            end,
        },
    }

    local moreOptionsDlg = LuaDialog(this)
    moreOptionsDlg.View = loadlayout(moreOptionsLayout)
    moreOptionsDlg.setTitle("More Options")
    moreOptionsDlg.setButton("Close", nil)
    moreOptionsDlg.show()
end

-- Function to open chat dialog
function openChatDialog()
    local chatLayout = {
        LinearLayout,
        orientation = "vertical",
        layout_width = "fill",
        layout_height = "fill",
        {
            ScrollView,
            layout_width = "fill",
            layout_height = "0dp",
            layout_weight = 1,
            {
                LinearLayout,
                id = "chatContainer",
                orientation = "vertical",
                layout_width = "fill",
                layout_height = "wrap",
                padding = "8dp",
            },
        },
        {
            LinearLayout,
            orientation = "horizontal",
            layout_width = "fill",
            layout_height = "wrap",
            {
                EditText,
                id = "userInput",
                layout_width = "0dp",
                layout_weight = 1,
                layout_height = "wrap",
                hint = "Type your question...",
            },
            {
                Button,
                text = "Send",
                layout_width = "wrap",
                layout_height = "wrap",
                onClick = function()
                    local inputText = userInput.getText().toString()
                    if inputText ~= "" then
                        addMessageToChat("You: " .. inputText, true)
                        sendMessageToAI(inputText)
                    else
                        service.speak("Please type a question before sending.")
                    end
                end,
            },
        },
    }

    local chatDlg = LuaDialog(this)
    chatDlg.View = loadlayout(chatLayout)
    chatDlg.setTitle("Chat with AI")
    chatDlg.setButton("Close", nil)
    chatDlg.show()
end

-- Function to add messages to chat
function addMessageToChat(message, isUser)
    local messageView = {
        TextView,
        text = message,
        layout_width = "wrap",
        layout_height = "wrap",
        textSize = 16,
        padding = "8dp",
        background = isUser and "#DDDDDD" or "#EEEEEE",
        textColor = isUser and "#000000" or "#333333",
    }
    local view = loadlayout(messageView)
    chatContainer.addView(view)
end

-- Function to send message to AI
function sendMessageToAI(prompt)
    local encodedPrompt = urlEncode(prompt)
    local fullUrl = apiUrl .. encodedPrompt

    Http.get(fullUrl, function(status, response)
        if status == 200 then
            local result = handleResponse(response)
            addMessageToChat("AI: " .. result, false)
        else
            addMessageToChat("AI: Error fetching response. Check your internet connection.", false)
        end
    end)
end

-- Function to handle AI response
function handleResponse(data)
    local decodedData = cjson.decode(data)
    if decodedData and decodedData.response then
        return decodedData.response
    else
        return "Sorry, I couldn't understand. Please try again."
    end
end

-- Function to show About dialog
function showAboutDialog()
    local aboutLayout = {
        LinearLayout,
        orientation = "vertical",
        layout_width = "fill",
        layout_height = "fill",
        {
            TextView,
            text = "This tool is developed by " .. developerName .. ".",
            layout_width = "fill",
            layout_height = "wrap",
            padding = "16dp",
            gravity = "center",
        },
        {
            Button,
            text = "Visit Telegram Channel",
            layout_width = "fill",
            layout_height = "wrap",
            onClick = function()
                local intent = Intent(Intent.ACTION_VIEW, Uri.parse("https://t.me/technicalblindnepal"))
                this.startActivity(intent)
            end,
        },
    }

    local aboutDlg = LuaDialog(this)
    aboutDlg.View = loadlayout(aboutLayout)
    aboutDlg.setTitle("About")
    aboutDlg.setButton("Close", nil)
    aboutDlg.show()
end

-- Function to show Settings dialog
function showSettingsDialog()
    local settingsLayout = {
        LinearLayout,
        orientation = "vertical",
        layout_width = "fill",
        layout_height = "fill",
        {
            CheckBox,
            id = "tickSoundCheckbox",
            text = "Enable Tick Sound",
            layout_width = "wrap",
            layout_height = "wrap",
            checked = tickSoundEnabled,
        },
        {
            CheckBox,
            id = "vibrationCheckbox",
            text = "Enable Vibration",
            layout_width = "wrap",
            layout_height = "wrap",
            checked = vibrationEnabled,
        },
        {
            Button,
            text = "Save Settings",
            layout_width = "fill",
            layout_height = "wrap",
            onClick = function()
                tickSoundEnabled = tickSoundCheckbox.isChecked()
                vibrationEnabled = vibrationCheckbox.isChecked()
                service.speak("Settings saved successfully!")
            end,
        },
    }

    local settingsDlg = LuaDialog(this)
    settingsDlg.View = loadlayout(settingsLayout)
    settingsDlg.setTitle("Settings")
    settingsDlg.setButton("Close", nil)
    settingsDlg.show()
end